"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const memfs_1 = require("memfs");
const path_1 = __importDefault(require("path"));
const asc_1 = __importDefault(require("assemblyscript/cli/asc"));
const loader_utils_1 = __importDefault(require("loader-utils"));
const schema_utils_1 = __importDefault(require("schema-utils"));
const option_1 = require("./option");
const loaderpkg = '@assemblyscript/loader';
const configuration = {
    name: 'assemblyscript-loader-loader',
};
const getInBundleWasmModule = (buf) => `
import loader from '${loaderpkg}';
export default async (imports) => {
  return await loader.instantiate(${buf.buffer}, imports || {});
};`;
const getFileWasmModule = (fetchUrl) => `
import loader from '${loaderpkg}';
export default async (imports) => {
  return await loader.instantiateStreaming(fetch(${fetchUrl}), imports || {});
}`;
exports.default = (content) => {
    const options = loader_utils_1.default.getOptions(content) || {};
    const callback = content.async();
    // eslint-disable-next-line
    const buildPath = content._compiler.outputPath;
    const output = path_1.default.join(buildPath, `${path_1.default.parse(content.resourcePath).name}.wasm`);
    const ascArgv = [
        path_1.default.relative(process.cwd(), content.resourcePath),
        '-o', path_1.default.relative(process.cwd(), output),
        ...option_1.transOptionToAscOption(options),
    ];
    schema_utils_1.default(option_1.schemaOptions, options, configuration);
    content.addDependency(loaderpkg);
    asc_1.default.ready.then(() => {
        asc_1.default.main(ascArgv, {
            readFile: (filename, baseDir) => {
                const filePath = path_1.default.join(baseDir, filename);
                console.log('filePath-read', filePath);
                return memfs_1.fs.readFileSync(filePath).toString();
            },
            writeFile: (filename, contents, baseDir) => {
                const filePath = path_1.default.join(baseDir, filename);
                console.log('filePath-write', filePath);
                memfs_1.fs.writeFileSync(filePath, contents);
            },
        }, (err) => {
            if (err) {
                callback(err);
                return 0;
            }
            const size = memfs_1.fs.statSync(output).size;
            const wasmFile = memfs_1.fs.readFileSync(output);
            // 默认 1000
            const limit = Number(options.limit) || 1000;
            if (size <= limit) {
                callback(null, getInBundleWasmModule(Buffer.from(wasmFile)));
            }
            else {
                const fileName = loader_utils_1.default.interpolateName(content, typeof options.name === 'string' && options.name
                    ? options.name
                    : '[name].[hash:7].wasm', {
                    context: content.rootContext,
                    content: wasmFile,
                });
                content.emitFile(fileName, wasmFile, undefined);
                let fetchUrl = `__webpack_public_path__ + ${JSON.stringify(fileName)}`;
                if (options.publicPath) {
                    fetchUrl = JSON.stringify(`${options.publicPath}${fileName}`);
                }
                else if (options.useInWorker) {
                    fetchUrl = `self.location.origin + '/${fileName}'`;
                }
                callback(null, getFileWasmModule(fetchUrl));
            }
            return 0;
        });
    });
};
//# sourceMappingURL=index.js.map